@@ -32,6 +32,9 @@ import org.apache.flink.cdc.common.types.DataTypeFamily;
 import org.apache.flink.cdc.common.types.DataTypeRoot;
 import org.apache.flink.cdc.common.types.DataTypes;
 import org.apache.flink.cdc.common.types.DecimalType;
+import org.apache.flink.cdc.common.types.LocalZonedTimestampType;
+import org.apache.flink.cdc.common.types.TimestampType;
+import org.apache.flink.cdc.common.types.ZonedTimestampType;
 
 import javax.annotation.Nullable;
 
@@ -175,6 +178,24 @@ public class SchemaUtils {
         if (lType.equals(rType)) {
             // identical type
             mergedType = rType;
+        } else if (lType instanceof TimestampType && rType instanceof TimestampType) {
+            return DataTypes.TIMESTAMP(
+                    Math.max(
+                            ((TimestampType) lType).getPrecision(),
+                            ((TimestampType) rType).getPrecision()));
+        } else if (lType instanceof ZonedTimestampType && rType instanceof ZonedTimestampType) {
+            return DataTypes.TIMESTAMP_TZ(
+                    Math.max(
+                            ((ZonedTimestampType) lType).getPrecision(),
+                            ((ZonedTimestampType) rType).getPrecision()));
+        } else if (lType instanceof LocalZonedTimestampType
+                && rType instanceof LocalZonedTimestampType) {
+            return DataTypes.TIMESTAMP_LTZ(
+                    Math.max(
+                            ((LocalZonedTimestampType) lType).getPrecision(),
+                            ((LocalZonedTimestampType) rType).getPrecision()));
+        } else if (lType.is(DataTypeFamily.TIMESTAMP) && rType.is(DataTypeFamily.TIMESTAMP)) {
+            return DataTypes.TIMESTAMP(TimestampType.MAX_PRECISION);
         } else if (lType.is(DataTypeFamily.INTEGER_NUMERIC)
                 && rType.is(DataTypeFamily.INTEGER_NUMERIC)) {
             mergedType = DataTypes.BIGINT();
@@ -184,7 +205,7 @@ public class SchemaUtils {
         } else if (lType.is(DataTypeFamily.APPROXIMATE_NUMERIC)
                 && rType.is(DataTypeFamily.APPROXIMATE_NUMERIC)) {
             mergedType = DataTypes.DOUBLE();
-        } else if (lType.is(DataTypeRoot.DECIMAL) && rType.is(DataTypeRoot.DECIMAL)) {
+        } else if (lType instanceof DecimalType && rType instanceof DecimalType) {
             // Merge two decimal types
             DecimalType lhsDecimal = (DecimalType) lType;
             DecimalType rhsDecimal = (DecimalType) rType;
@@ -193,25 +214,21 @@ public class SchemaUtils {
                             lhsDecimal.getPrecision() - lhsDecimal.getScale(),
                             rhsDecimal.getPrecision() - rhsDecimal.getScale());
             int resultScale = Math.max(lhsDecimal.getScale(), rhsDecimal.getScale());
+            Preconditions.checkArgument(
+                    resultIntDigits + resultScale <= DecimalType.MAX_PRECISION,
+                    String.format(
+                            "Failed to merge %s and %s type into DECIMAL. %d precision digits required, %d available",
+                            lType,
+                            rType,
+                            resultIntDigits + resultScale,
+                            DecimalType.MAX_PRECISION));
             mergedType = DataTypes.DECIMAL(resultIntDigits + resultScale, resultScale);
-        } else if (lType.is(DataTypeRoot.DECIMAL) && rType.is(DataTypeFamily.EXACT_NUMERIC)) {
+        } else if (lType instanceof DecimalType && rType.is(DataTypeFamily.EXACT_NUMERIC)) {
             // Merge decimal and int
-            DecimalType lhsDecimal = (DecimalType) lType;
-            mergedType =
-                    DataTypes.DECIMAL(
-                            Math.max(
-                                    lhsDecimal.getPrecision(),
-                                    lhsDecimal.getScale() + getNumericPrecision(rType)),
-                            lhsDecimal.getScale());
-        } else if (rType.is(DataTypeRoot.DECIMAL) && lType.is(DataTypeFamily.EXACT_NUMERIC)) {
+            mergedType = mergeExactNumericsIntoDecimal((DecimalType) lType, rType);
+        } else if (rType instanceof DecimalType && lType.is(DataTypeFamily.EXACT_NUMERIC)) {
             // Merge decimal and int
-            DecimalType rhsDecimal = (DecimalType) rType;
-            mergedType =
-                    DataTypes.DECIMAL(
-                            Math.max(
-                                    rhsDecimal.getPrecision(),
-                                    rhsDecimal.getScale() + getNumericPrecision(lType)),
-                            rhsDecimal.getScale());
+            mergedType = mergeExactNumericsIntoDecimal((DecimalType) rType, lType);
         } else {
             throw new IllegalStateException(
                     String.format("Incompatible types: \"%s\" and \"%s\"", lType, rType));
@@ -224,6 +241,20 @@ public class SchemaUtils {
         }
     }
 
+    private static DataType mergeExactNumericsIntoDecimal(
+            DecimalType decimalType, DataType otherType) {
+        int resultPrecision =
+                Math.max(
+                        decimalType.getPrecision(),
+                        decimalType.getScale() + getNumericPrecision(otherType));
+        Preconditions.checkArgument(
+                resultPrecision <= DecimalType.MAX_PRECISION,
+                String.format(
+                        "Failed to merge %s and %s type into DECIMAL. %d precision digits required, %d available",
+                        decimalType, otherType, resultPrecision, DecimalType.MAX_PRECISION));
+        return DataTypes.DECIMAL(resultPrecision, decimalType.getScale());
+    }
+
     @VisibleForTesting
     public static int getNumericPrecision(DataType dataType) {
         if (dataType.is(DataTypeFamily.EXACT_NUMERIC)) {
@@ -355,6 +386,25 @@ public class SchemaUtils {
         return oldSchema.copy(columns);
     }
 
+    /**
+     * This function determines if the given schema change event {@code event} should be sent to
+     * downstream based on if the given transform rule has asterisk, and what columns are
+     * referenced.
+     *
+     * <p>For example, if {@code hasAsterisk} is false, then all {@code AddColumnEvent} and {@code
+     * DropColumnEvent} should be ignored since asterisk-less transform should not emit schema
+     * change events that change number of downstream columns.
+     *
+     * <p>Also, {@code referencedColumns} will be used to determine if the schema change event
+     * affects any referenced columns, since if a column has been projected out of downstream, its
+     * corresponding schema change events should not be emitted, either.
+     *
+     * <p>For the case when {@code hasAsterisk} is true, things will be cleaner since we don't have
+     * to filter out any schema change events. All we need to do is to change {@code
+     * AddColumnEvent}'s inserting position, and replacing `FIRST` / `LAST` with column-relative
+     * position indicators. This is necessary since extra calculated columns might be added, and
+     * `FIRST` / `LAST` position might differ.
+     */
     public static Optional<SchemaChangeEvent> transformSchemaChangeEvent(
             boolean hasAsterisk, List<String> referencedColumns, SchemaChangeEvent event) {
         Optional<SchemaChangeEvent> evolvedSchemaChangeEvent = Optional.empty();
