@@ -141,9 +141,21 @@ CREATE TABLE precision_types
     datetime_c           DATETIME(0),
     datetime3_c          DATETIME(3),
     datetime6_c          DATETIME(6),
-    timestamp_c          TIMESTAMP(0),
-    timestamp3_c         TIMESTAMP(3),
-    timestamp6_c         TIMESTAMP(6),
+    timestamp_c          TIMESTAMP(0) NULL,
+    timestamp3_c         TIMESTAMP(3) NULL,
+    timestamp6_c         TIMESTAMP(6) NULL,
+    float_c0             FLOAT(6, 0),
+    float_c1             FLOAT(20, 3),
+    float_c2             FLOAT(24, 12),
+    real_c0              REAL(6, 0),
+    real_c1              REAL(20, 3),
+    real_c2              REAL(24, 12),
+    double_c0            DOUBLE(6, 0),
+    double_c1            DOUBLE(20, 3),
+    double_c2            DOUBLE(24, 12),
+    double_precision_c0  DOUBLE PRECISION(6, 0),
+    double_precision_c1  DOUBLE PRECISION(20, 3),
+    double_precision_c2  DOUBLE PRECISION(24, 12),
     PRIMARY KEY (id)
 ) DEFAULT CHARSET=utf8;
 
@@ -160,4 +172,16 @@ VALUES (DEFAULT,
         '2020-07-17 18:00:22',
         '2020-07-17 18:00',
         '2020-07-17 18:00:22',
-        '2020-07-17 18:00:22');
+        '2020-07-17 18:00:22',
+        2,
+        3,
+        5,
+        7,
+        11,
+        13,
+        17,
+        19,
+        23,
+        29,
+        31,
+        37);
