@@ -376,7 +376,9 @@ public class SchemaRegistryRequestHandler implements Closeable {
                                                                                     .getType()
                                                                                     .nullable(),
                                                                             col.getAddColumn()
-                                                                                    .getComment())))
+                                                                                    .getComment(),
+                                                                            col.getAddColumn()
+                                                                                    .getDefaultValueExpression())))
                                             .collect(Collectors.toList())));
                 }
             case DROP_COLUMN:
@@ -428,7 +430,9 @@ public class SchemaRegistryRequestHandler implements Closeable {
                                                         Column.physicalColumn(
                                                                 value,
                                                                 column.getType().nullable(),
-                                                                column.getComment())));
+                                                                column.getComment(),
+                                                                column
+                                                                        .getDefaultValueExpression())));
                                     });
 
                     List<SchemaChangeEvent> events = new ArrayList<>();
